#------------------------------------------------------------------------------
# TUAM Admin console deployment/undeployment script
#
# IBM Tivoli Usage and Accounting Manager
# AUC
# (C) Copyright IBM Corp. 2007
#
# Installing the console:
# wsadmin.sh -f deployTUAMConsole.py install
#
# Uninstalling the console:
# wsadmin.sh -f deployTUAMConsole.py remove
#------------------------------------------------------------------------------
import sys
import os
import os.path
import re
from java.lang import Runtime
from java.io import StringReader
import org.xml.sax as sax
import org.apache.xerces.parsers.DOMParser as xmlparser
import zipfile
import shutil
import glob

# Because jython treats Windows like a Unix system, os.environ fails. Workaround
# by forcing it to use a different method
try:
	import javaos
	if javaos._osType == 'posix' and java.lang.System.getProperty('os.name').startswith('Windows'):
		sys.registry.setProperty('python.os', 'nt')
		reload(javaos)
		os.environ = javaos._shellEnv._getEnvironment()
except:
	pass
	
#------------------------------------------------------------------------------
# Get the directory, as a string, where WAS is installed.
#------------------------------------------------------------------------------
def getWASHome(cell, node):
	varMap = AdminConfig.getid("/Cell:" + cell + "/Node:" + node + "/VariableMap:/")
	entries = AdminConfig.list("VariableSubstitutionEntry", varMap)
	eList = entries.splitlines()
	for entry in eList:
		name =  AdminConfig.showAttribute(entry, "symbolicName")
		print name
		if name == "WAS_INSTALL_ROOT":
			value = AdminConfig.showAttribute(entry, "value")
			return value
	#failover
	return java.lang.System.getenv('WAS_HOME')


#------------------------------------------------------------------------------
# Get the file separator character
#
# This gets the file separator for the node in which we plan to install the
# console.  Therefore we can't just use the value that python on Java gives
# us.  Instead, check the platform of the node, and use "\" for windows and
# "/" for everything else.
#------------------------------------------------------------------------------
def getFileSep(node):
	os = AdminTask.getNodePlatformOS("-nodeName " + node)
	if os == 'windows':
		return '\\'
	else:
		return '/'

#------------------------------------------------------------------------------
# setCellVar
#------------------------------------------------------------------------------
def setCellVar(cell):
	try:
		varMap = AdminConfig.getid("/Cell:" + cell + "/VariableMap:/")
		prop = [[['symbolicName', 'WAS_CELL_NAME'], ['value', cell]]]
		AdminConfig.modify(varMap, [['entries', prop]])
		return 1
	except:
		print "Error during setCellVar(" + cell + "):", sys.exc_info()
		return 0



#------------------------------------------------------------------------------
# deployTUAMAdminConsole
# Deploy the aucConsole.war using the AdminApp update command, and then map it
# to the admin_host virtual host.
#------------------------------------------------------------------------------
def deployTUAMAdminConsole (cell, node, server, type):
	warFile  = os.path.expandvars('${TUAM_HOME}/setup/console/aucConsoleAUIML.war')
	try:
		print "Deploying aucConsole.war"
		AdminApp.update('isc' , 'modulefile', ['-operation', 'addupdate', '-contents', warFile, '-contenturi', 'aucConsole.war', '-usedefaultbindings', '-contextroot' , '/tuamConsole', '-MapWebModToVH', [['.*', '.*', 'default_host']]])

	except:
		error = str(sys.exc_info()[1])
		if error.count("7279E") > 0: # catch WASX7279E (app with given name already exists)
			print "TUAM Administration console is already installed."
		else:
			print "Exception occurred during deployTuamConsole(" + cell +", " + node + ", " + server + "):", error
		return 0
	return 1

def addJarToWar (jar, jardir):
	try:
		print "Adding " + jar
		AdminApp.update('isc', 'file', ['-operation', 'addupdate', '-contents', jardir + '/' + jar, '-contenturi', 'aucConsole.war/WEB-INF/lib/' + jar])

	except:
		error = str(sys.exc_info()[1])
		if error.count("7279E") > 0: # catch WASX7279E (app with given name already exists)
			print "Collector jar is already installed."
		else:
			print "Exception occurred during collector jar registration:", error
		return 0
	return 1

#------------------------------------------------------------------------------
# Get a tuple containing the cell, node, server name, and type
#------------------------------------------------------------------------------
def getCellNodeServer():
	servers = AdminConfig.list("Server").splitlines()
	for serverId in servers:
		serverName = serverId.split("(")[0]
		server = serverId.split("(")[1]  #remove name( from id
		server = server.split("/")
		cell = server[1]
		node = server[3]
		cellId = AdminConfig.getid("/Cell:" + cell + "/")
		cellType = AdminConfig.showAttribute(cellId, "cellType")
		if cellType == "DISTRIBUTED":
			if AdminConfig.showAttribute(serverId, "serverType") == "DEPLOYMENT_MANAGER":
				return (cell, node, serverName, "DEPLOYMENT_MANAGER")
		elif cellType == "STANDALONE":
			if AdminConfig.showAttribute(serverId, "serverType") == "APPLICATION_SERVER":
				return (cell, node, serverName, "APPLICATION_SERVER")
	return None


#------------------------------------------------------------------------------
# Print script usage
#------------------------------------------------------------------------------
def printUsage():
	print "Usage: wsadmin deployTUAMConsole.py install"
	print "  or:  wsadmin deployTUAMConsole.py remove"
	print ""

#------------------------------------------------------------------------------
# Deployment Engine class
#------------------------------------------------------------------------------
class DE:

	class iuInstance:

		def __init__(self, node):
			self.node = node
			self.attrib = {}
			for element in ['instanceID', 'UUID', 'identityName', 'version', 'hostingEnv', 'discriminant']:
				self.attrib[element] = node.getElementsByTagName(element).item(0).getChildNodes().item(0).nodeValue

		def __getitem__(self, item):
			return self.attrib[item]


	def __init__(self):
		self.si_path = None
		self.ext = None
		self.document = None
		self.inventory = []

		self._findDE()
		if self.si_path == None:
			raise Exception('Cannot locate DE. Please set the SI_PATH environment variable.')

		inventory = Runtime.getRuntime().exec([os.path.join(self.si_path, 'de_lsrootiu.' + self.ext)]).inputStream
		parser = xmlparser()
		parser.reset()
		parser.parse(sax.InputSource(inventory))
		self.document = parser.getDocument()
		instances = self.document.getElementsByTagName('iuInstance')
		for i in range(instances.getLength()):
			self.inventory.append(DE.iuInstance(instances.item(i)))

	def _findDE(self):
		try:
			si_path = os.environ['SI_PATH']
		except:
			si_path = ""
		home = os.path.expanduser('~')
		if java.lang.System.getProperty('os.name').startswith('Windows'):
			ext = 'cmd'
		else:
			ext = 'sh'
		for dir in [os.path.join(si_path, 'bin'), home + '/' + os.path.expandvars('.acsi_${USER}/bin'), '/usr/ibm/common/acsi/bin', os.path.expandvars('%USERPROFILE%\\acsi_%USERNAME%'), os.path.expandvars('%PROGRAMFILES(x86)%\\IBM\\Common\\acsi\\bin'), os.path.expandvars('%PROGRAMFILES%\\IBM\\Common\\acsi\\bin')]:
			if os.path.exists(os.path.join(dir, 'de_lsrootiu.' + ext)):
				self.si_path = dir
				self.ext = ext
				return

	def lookupVersion(self, iu, path=None):
		for instance in self.inventory:
			if instance['identityName'] == iu and path == None or path == instance['discriminant']:
				return instance['version']

		return None


#------------------------------------------------------------------------------
# Install the admin console
#------------------------------------------------------------------------------
def doInstall(jardir):
	de = DE()
	tipver = de.lookupVersion('TIPCore', os.environ['TIP_HOME'])
	auiml = '8.1.0.6' # Default for unknown version
	if tipver.startswith('2.1'):
		auiml = '8.1.0.4'
	elif tipver.startswith('2.2.0.5'):
		auiml = '8.1.0.6'
	else:
		print "WARNING: Unsupported TIP version, defaulting to latest AUIML available"
	
	print "Detected TIP version " + tipver + " -> using AUIML version " + auiml

	topology = getCellNodeServer()
	if topology == None:
		sys.stderr.write("Could not find suitable server\n")
		if failOnError == "true":
			sys.exit(105)
	else:
		(cell, node, server, type) = topology[0:4]
		

		# Take a copy of the base war (zipfile append doesn't work properly in Jython)
		src = os.path.expandvars('${TUAM_HOME}/setup/console/aucConsole.war')
		srczip = zipfile.ZipFile(src)
		war = os.path.expandvars('${TUAM_HOME}/setup/console/aucConsoleAUIML.war')
		warzip = zipfile.ZipFile(war, 'w', zipfile.ZIP_STORED) # Should be faster as the jars are already compressed
		count = 0
		for file in srczip.infolist():
			if not file.filename.startswith('auiml'):
				warzip.writestr(file, srczip.read(file.filename))
			else:
				if file.filename[6:13] == auiml:
					ofilename = file.filename
					file.filename = file.filename[14:]
					if file.filename != "WEB-INF/" and file.filename != "":
						warzip.writestr(file, srczip.read(ofilename))

		srczip.close()
		# Add the collector jars if installed
		for jar in ['aucBCP.jar', 'aucECP.jar', 'aucNFC.jar']:
			if os.path.exists(os.path.join(jardir, jar)):
				print "Adding " + jar
				warzip.write(os.path.join(jardir, jar), "WEB-INF/lib/" + jar)
		# Add the correct version of AUIML to the war
		#print "Adding AUIML"
		#auimlzip = zipfile.ZipFile(os.path.expandvars('${TUAM_HOME}/setup/console/auiml' + auiml + '.zip'))
		#for file in auimlzip.infolist():
		#	warzip.writestr(file, auimlzip.read(file.filename))
		#auimlzip.close()
		warzip.close()

		retVal = deployTUAMAdminConsole(cell, node, server, type)
		if retVal == 1:
			AdminConfig.save()
			addShLib("RXA_SH_LIB")
		else:
			print "Skipping Config Save"
			if failOnError == "true":
				sys.exit(109)


#------------------------------------------------------------------------------
# Uninstall the admin console
#------------------------------------------------------------------------------
def doRemove():

	AdminApp.update('isc' , 'modulefile', ['-operation', 'delete', '-contenturi', 'aucConsole.war'])
	AdminConfig.save()

#------------------------------------------------------------------------------
# Add a shared library to the war
#------------------------------------------------------------------------------
def addShLib(shLibName):
	deployed = AdminConfig.getid('/Deployment:isc/')
	print "ISC deployment: " + deployed
	appDeployed = AdminConfig.showAttribute( deployed, 'deployedObject')
	print "Application attributes for isc: " + appDeployed
	webModules = AdminConfig.list('WebModuleDeployment', appDeployed ).split(lineSeparator)
	for webModule in webModules:
		uri = AdminConfig.showAttribute(webModule, 'uri')
		if ( uri == "aucConsole.war" ) :
			AdminConfig.modify( webModule, [['classloaderMode', 'PARENT_FIRST']] )
			classLoaderId = AdminConfig.create( 'Classloader', webModule, [['mode', 'PARENT_FIRST']] )
			AdminConfig.save()
			print "ClassloaderID : " + classLoaderId
			libraryRef = AdminConfig.create('LibraryRef', classLoaderId, [['libraryName', shLibName ],  ['sharedClassloader', 'true']])
			print "LibraryRef: " + libraryRef
			AdminConfig.save()

#------------------------------------------------------------------------------
# Main entry point
#------------------------------------------------------------------------------

failOnError = "false"
jardir = None

if len(sys.argv) < 1 or len(sys.argv) > 2:
	sys.stderr.write("Invalid number of arguments\n")
	printUsage()
	sys.exit(101)
else:
	if len(sys.argv) == 2:
		if sys.argv[1] == "-failonerror":
			failOnError = "true"
			print "failonerror is enabled"
		else:
			jardir = sys.argv[1]

	mode = sys.argv[0]
	if mode == "remove" or mode == "replace":
		print "Removing TUAM Administration Console..."
		doRemove()
	if mode == "install" or mode == "replace":
		print "Installing TUAM Administration Console..."
		doInstall(jardir)
		# Copy reporting javascript library files if Cognos directory exists
		cogdir = os.path.join(os.environ['TIP_HOME'], "profiles", "TIPProfile", "installedApps", "TIPCell", "IBM Cognos 8.ear", "p2pd.war", "tivoli")
		if os.path.exists(cogdir):
			for srcfile in glob.glob(os.path.join(os.environ['TUAM_HOME'], "setup", "console", "reportscognos", "jslib", "*")):
				shutil.copy(srcfile, os.path.join(cogdir, os.path.basename(srcfile)))
			
	if mode not in ["install", "remove", "replace"]:
		sys.stderr.write("Invalid command:  " + mode + "\n")
		printUsage()
		if failOnError == "true":
			sys.exit(103)



